{ :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  :: QuickReport 3.0 for Delphi 3 or newer                   ::
  ::                                                         ::
  :: Example MDI with Custom preview project                 ::
  ::                                                         ::
  :: Copyright (c) 1998 QuSoft AS                            ::
  :: All Rights Reserved                                     ::
  ::                                                         ::
  :: web: http://www.qusoft.no   fax: +47 22 41 74 91        ::
  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: }
unit prev2_32;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls,qrprntr,quickrpt;

type
  TfrmNetScapePreview = class(TForm)
    Panel1: TPanel;
    QRPreview1: TQRPreview;
    btnPrint: TButton;
    btnNextPage: TButton;
    btnPrevPage: TButton;
    btnZoomIn: TButton;
    btnZoomOut: TButton;
    btnClose: TButton;
    btnOpen: TButton;
    procedure btnPrintClick(Sender: TObject);
    procedure btnPrevPageClick(Sender: TObject);
    procedure btnNextPageClick(Sender: TObject);
    procedure btnZoomInClick(Sender: TObject);
    procedure btnZoomOutClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);
    procedure QRPreview1PageAvailable(Sender: TObject; PageNum: Integer);
    procedure FormCreate(Sender: TObject);
    procedure btnOpenClick(Sender: TObject);
  private
    { Private declarations }
    FQRPrinter : TQRPrinter;
    procedure Init;
    procedure UpdateButtons;
  public
    { Public declarations }
    pQuickreport : TQuickRep;
    bPleaseInit : Boolean;
    bCanPrint: boolean;
    sTitle : string;
    constructor CreatePreview(AOwner : TComponent; aQRPrinter : TQRPrinter);
    property QRPrinter : TQRPrinter read FQRPrinter write FQRPrinter;
  end;

function TempFilename : string;

var
  frmNetScapePreview: TfrmNetScapePreview;

implementation

uses qrselect, qrqrp;

{$R *.DFM}

function TempFilename : string;
var
  AName,
  ADir : array[0..80] of char;
begin
  GetTempPath(30, adir);
  GetTempFilename(aDir, PChar('QRP'), 0, aName);
  result := StrPas(aName);
end;

constructor TfrmNetScapePreview.CreatePreview(AOwner : TComponent; aQRPrinter : TQRPrinter);
begin
  inherited Create(AOwner);
  QRPrinter := aQRPrinter;
  QRPreview1.QRPrinter := aQRPrinter;
  if (QRPrinter <> nil) and (QRPrinter.Title <> '') then Caption := QRPrinter.Title;
end;

procedure TfrmNetScapePreview.UpdateButtons;
begin
  with QRPreview1 do
  begin
    btnPrevPage.Enabled := PageNumber > 1;
    btnNextPage.Enabled := PageNumber < QRPreview1.QRPrinter.PageCount;
  end;
end;

procedure TfrmNetScapePreview.btnPrintClick(Sender: TObject);
var
  SrcFileName, DestFileName, CopyFileName: string;
  i: integer;
  RepeatPrint: boolean;
  CopyList: TStringList;
begin
  // When you have a QRP file, you can't use
  // the printersetup dialog.  In this case
  // we use a simple dialog to allow the user
  // to select the printer and set the page range

  // This dialog is very simple, you can pick the printer
  // the range of pages.  You could easily add other features
  // like prompting for the number copies.
  with TfrmSelectPrinter.Create(application) do
  begin
    QRPrinter := self.QRPrinter;
    PrinterIndex := qrprinter.printerindex;
    ShowModal;
    if ModalResult = mrOk then
    begin
      qrprinter.printerindex := cbprinters.ItemIndex;

      // If the user selected a range of pages, then
      // we do some extra work.  We save the rendered report
      // to a QRP file and then call a function that will
      // copy the range of pages to a new QRP file.  We then
      // print that file.
      if (FromPage > 0) or (qrprinter.master = nil) then
      begin
        SrcFileName := TempFileName;
        DestFileName := TempFileName;
        CopyFileName := TempFileName;

        // Save the report to a file so that we can work with it.
        qrprinter.Save(SrcFileName);

        // Build a new file with the range of pages that was selected
        RangeFromQRP(SrcFileName, DestFileName, FromPage, ToPage);

        with TQRPrinter.Create do
        begin
          printerindex := cbprinters.ItemIndex;

          // If we want more than 1 copy, we just repeat the pages in the
          // list.  This will make sure that only a single print job is
          // issued.
          if NumCopies > 1 then begin
            CopyList := TStringList.Create;

            for i := 1 to NumCopies do
              CopyList.Add(DestFileName);

            CombineQRP(CopyFileName, CopyList);

            CopyList.Free;

            Load(CopyFileName);
          end
          else
            Load(DestFileName);

            Print;
          Free;
        end;
        DeleteFile(CopyFileName);
        DeleteFile(DestFileName);
        DeleteFile(SrcFileName);
      end
      else begin
        TQuickRep(qrprinter.master).printersettings.printerindex := cbprinters.ItemIndex;

        // If we are rendering the report, we check to see the printer driver
        // supports multiple copies.  If it does, we set the
        // report's PrinterSettings.copies property.  Otherwise, we just print
        // it over and over again.
        if NumCopies > 1 then begin
          qrprinter.Copies := NumCopies;

          RepeatPrint := qrprinter.Copies = 1;
        end
        else
          RepeatPrint := false;

        if RepeatPrint then begin
          for I := 1 to NumCopies do
            qrprinter.Print;
        end
        else begin
          TQuickRep(qrprinter.master).PrinterSettings.Copies := NumCopies;
          qrprinter.print;
        end;
      end;
    end;
    free;
  end;
end;

procedure TfrmNetScapePreview.btnPrevPageClick(Sender: TObject);
begin
  Application.ProcessMessages;
  with QRPreview1 do
    if PageNumber > 1 then
      PageNumber := PageNumber - 1;
  UpdateButtons;
end;

procedure TfrmNetScapePreview.btnNextPageClick(Sender: TObject);
begin
  Application.ProcessMessages;

  with QRPreview1 do
    if PageNumber < QRPreview1.QRPrinter.PageCount then
      PageNumber := PageNumber + 1;
  UpdateButtons;
end;

procedure TfrmNetScapePreview.btnZoomInClick(Sender: TObject);
begin
  Application.ProcessMessages;
  with QRPreview1 do
    Zoom := Zoom + 10;
end;

procedure TfrmNetScapePreview.btnZoomOutClick(Sender: TObject);
begin
  Application.ProcessMessages;
  with QRPreview1 do
    if Zoom > 10 then
      Zoom := Zoom - 10;
end;

procedure TfrmNetScapePreview.btnCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmNetScapePreview.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  QRPrinter.ClosePreview(Self);
  Action := caFree;
end;

procedure TfrmNetScapePreview.FormShow(Sender: TObject);
begin
  { Signal ourselves that we need to update the form }
  bPleaseInit := True;
end;

procedure TfrmNetScapePreview.Init;
var
  OffSet: integer;
begin
  if bPleaseInit then
  begin
    { Force the preview to come up in the zoom setting that we want}
    QRPreview1.ZoomToFit;

    { If the caller reports want to disable printing, then it will set}
    { the following boolean to false }
    if not bCanPrint then
    begin
      btnPrint.Enabled := bCanPrint;
      btnPrint.Visible := bCanPrint;

      // Shift the rest of the buttons over to fill the space of the hidden control
      OffSet := btnNextPage.Left - btnPrint.Left;
      btnNextPage.Left := btnNextPage.Left - Offset;
      btnPrevPage.Left := btnPrevPage.Left - Offset;
      btnZoomIn.Left := btnZoomIn.Left - Offset;
      btnZoomOut.Left := btnZoomOut.Left - Offset;
      btnClose.Left := btnClose.Left - Offset;
    end;
    UpdateButtons;

    bPleaseInit := False;
  end;
end;

procedure TfrmNetScapePreview.QRPreview1PageAvailable(Sender: TObject;
  PageNum: Integer);
begin
  Init;

  if PageNum = 1 then
    Caption := QRPreview1.QRPrinter.Title + ' - 1 page'
  else
    Caption := QRPreview1.QRPrinter.Title + ' - ' + IntToStr(PageNum) + ' pages';

  case QRPreview1.QRPrinter.Status of
    mpReady: Caption := Caption + ' READY';
    mpBusy: Caption := Caption + ' BUSY';
    mpFinished: Caption := Caption + ' FINISHED';
  end;
  UpdateButtons;
end;

procedure TfrmNetScapePreview.FormCreate(Sender: TObject);
begin
  { We can disable functionality at runtime }
  bCanPrint := True;
end;

procedure TfrmNetScapePreview.btnOpenClick(Sender: TObject);
begin
  with TOpenDialog.Create(Application) do
  try
    Title := 'Load report';
    Filter := 'QuickReport file (*.' +cQRPDefaultExt + ')|*.' + cqrpDefaultExt;
    if Execute then
      if FileExists(FileName) then
      begin
        qrprinter.master := nil;
        QRPrinter.Load(Filename);
        QRPreview1.PageNumber := 1;
        QRPreview1.PreviewImage.PageNumber := 1;
        UpdateButtons;
      end
      else
        ShowMessage('Error!  File does not exist');
  finally
    free;
  end;
end;

end.
