{ :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  :: QuickReport 3.0 for Delphi 3 or newer                   ::
  ::                                                         ::
  :: Example printer setup for Custom preview project        ::
  ::                                                         ::
  :: Copyright (c) 1998 QuSoft AS                            ::
  :: All Rights Reserved                                     ::
  ::                                                         ::
  :: web: http://www.qusoft.no   fax: +47 22 41 74 91        ::
  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: }
unit qrselect;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, printers, qrprntr, ComCtrls, Spin, ExtCtrls;

type
  TfrmSelectPrinter = class(TForm)
    Label1: TLabel;
    cbPrinters: TComboBox;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    BitBtn3: TBitBtn;
    GroupBox1: TGroupBox;
    AllPagesRb: TRadioButton;
    RangeRb: TRadioButton;
    Label2: TLabel;
    Label3: TLabel;
    FromEdt: TEdit;
    ToEdt: TEdit;
    GroupBox2: TGroupBox;
    Label4: TLabel;
    CopiesSe: TSpinEdit;
    Bevel1: TBevel;
    StatusLbl: TLabel;
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure BitBtn3Click(Sender: TObject);
    procedure AllPagesRbClick(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
    procedure FromEdtChange(Sender: TObject);
    procedure cbPrintersChange(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    NumCopies: integer;
    Printer : TPrinter;
    PrinterIndex: integer;
    QRPrinter: TQRPrinter;
    FromPage, ToPage: integer;
  end;

var
  frmSelectPrinter: TfrmSelectPrinter;

implementation

{$R *.DFM}

procedure TfrmSelectPrinter.FormDestroy(Sender: TObject);
begin
  // All done, free the printer object
  printer.free;
end;

procedure TfrmSelectPrinter.FormShow(Sender: TObject);
var
  nIdx: integer;
begin
  StatusLbl.Caption := 'Please wait...';
  // create a printer to get the list of printers
  Printer := TPrinter.Create;

  if NumCopies < 1 then
    NumCopies := 1;

  CopiesSe.Value := NumCopies;

  // Now populate the combobox with the list of printer names
  try
    with Printer do
      for nIdx := 0 to Printers.Count - 1 do
        cbPrinters.Items.Add(printers[nIdx]);
  except
    raise;
  end;

  // Set the combo box to the current printer as defined
  // by the caller.  It may not be the default printer
  cbPrinters.ItemIndex := PrinterIndex;

  // Update the page count
  AllPagesRb.Caption := 'All ' + IntToStr(QRPrinter.PageCount) + ' pages';
  FromEdt.Text := '1';
  ToEdt.Text := IntToStr(QRPrinter.PageCount);

  application.ProcessMessages;
  cbPrintersChange(Sender);
end;

procedure TfrmSelectPrinter.BitBtn3Click(Sender: TObject);
begin
  // reset the printer index back to the default
  Printer.PrinterIndex := -1;
  cbPrinters.ItemIndex := Printer.PrinterIndex;
  cbPrintersChange(Sender);
end;

procedure TfrmSelectPrinter.AllPagesRbClick(Sender: TObject);
begin
  FromEdt.Enabled := RangeRb.Checked;
  ToEdt.Enabled := FromEdt.Enabled;
end;

procedure TfrmSelectPrinter.BitBtn1Click(Sender: TObject);
begin
  FromPage := 0;
  ToPage := 0;

  NumCopies := CopiesSe.Value;

  if RangeRb.Checked then
  begin
    try
      FromPage := StrToInt(FromEdt.text);
      ToPage := StrToInt(ToEdt.text);
    finally
    end;
  end;
end;

procedure TfrmSelectPrinter.FromEdtChange(Sender: TObject);
const
  sIntKeys = '0123456789';
var
  nIdx: Integer;
begin
  with Sender as TEdit do
  begin
    nIdx := StrLen(pchar(Text));

    // only allow numbers to be entered
    for nIdx := 1 to nIdx do
      if StrScan(sIntKeys, char(Text[nIdx])) = nil then
      begin
        Text := '';
        break;
      end;
  end;
end;

procedure TfrmSelectPrinter.cbPrintersChange(Sender: TObject);
var
  SaveCopies,
  SaveIndex: integer;
begin
  with QRPrinter do
  begin
    SaveIndex := PrinterIndex;
    PrinterIndex := cbPrinters.ItemIndex;

    // Quick and easy way to see if the printer supports the printing
    // of multiple copies
    SaveCopies := Copies;
    Copies := 2;
    if Copies > 1 then
      StatusLbl.Caption := 'Printer supports multiple copies'
    else
      StatusLbl.Caption := 'Printer does not supports multiple copies';

    Copies := SaveCopies;
    PrinterIndex := SaveIndex;
  end;
end;

end.
